@@ -8,7 +8,7 @@ use strict;
 use warnings;
 use base qw( Circle::Net );
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 require Circle;
 Circle->VERSION( '0.142470' );  # require late-loading of Tangence::Class
@@ -62,7 +62,7 @@ sub DECLARE_TANGENCE
    Circle::Net::Matrix::Room->DECLARE_TANGENCE;
 
    require Net::Async::Matrix;
-   Net::Async::Matrix->VERSION( '0.18002' );
+   Net::Async::Matrix->VERSION( '0.18003' );
 }
 
 sub WEAKSELF_EVAL
@@ -205,6 +205,12 @@ __PACKAGE__->APPLY_Setting( access_token =>
    type        => 'str',
 );
 
+__PACKAGE__->APPLY_Setting( initial_backlog =>
+   description => "Initial size of backlog to request per room",
+   type        => "int",
+   default     => 100,
+);
+
 sub command_connect
    : Command_description("Connect to the homeserver")
    : Command_arg('homeserver?')
@@ -216,7 +222,8 @@ sub command_connect
 
    my $matrix = $self->{matrix};
    $matrix->configure(
-      server => $homeserver,
+      server           => $homeserver,
+      first_sync_limit => $self->{initial_backlog},
    );
 
    # TODO: would be nice if Circle could cope with Future-returning
