@@ -8,14 +8,16 @@ use strict;
 use warnings;
 use base qw( Tangence::Object Circle::WindowItem );
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Data::Dump qw( pp );
 use Scalar::Util qw( weaken );
 
-use Net::Async::Matrix::Utils qw( parse_formatted_message );
+use Net::Async::Matrix::Utils qw( parse_formatted_message build_formatted_message );
 use Circle::TaggedString;
 
+use Circle::Net::Matrix::Utils qw( parse_markdownlike );
+
 # To allow for out-of-tree development, use an inline Tangence class
 # declaration instead of a .tan file
 #
@@ -151,8 +153,6 @@ sub on_synced_state
 
    $self->set_prop_name( $name );
    $self->set_prop_tag( $name );
-
-   $self->SUPER::set_prop_level( 0 );
 }
 
 sub on_state_name
@@ -178,11 +178,29 @@ sub _generate_roomname
       return $event->{content}{name};
 
    my $myself = $self->{net}{matrix}->myself;
+   my $domain = ( $myself->user_id =~ m/:(.*)$/ )[0];
+
+   # TODO: pick canonical alias if there is one
+
+   my %aliasmap = %{ $state->get_events( 'm.room.aliases' ) };
+
+   # Prefer an alias defined by my own homeserver
+   my $local_aliases = $aliasmap{$domain};
+   $local_aliases and
+      return $local_aliases->{content}{aliases}[0];
+
+   # TODO: try to stable-sort them
+   my @aliases = map { @{ $_->{content}{aliases} } }
+                 values %aliasmap;
+
+   return $aliases[0] if @aliases;
+
    my @others = grep { $_->user->user_id ne $myself->user_id } $state->members;
 
    return $others[0]->displayname if @others == 1;
 
-   return "TODO";
+   warn "Did not find exactly one peer for ${\ $self->{room}->room_id }; bailing out\n";
+   return $self->{room}->room_id;
 }
 
 sub on_state_topic
@@ -300,20 +318,40 @@ sub on_membership
    if( my $membership = $changes{membership} ) {
       if( ( $membership->[0]//"") ne "join" and $membership->[1] eq "join" ) {
          $self->push_displayevent( "matrix.join" => {
-            name    => $member->displayname // "[$member_id]",
+            name    => $member->displayname,
             user_id => $member_id,
          }, time => $tstamp );
          $self->bump_level( 1 );
-         return
       }
-      if( $membership->[0] eq "join" and !defined $membership->[1] ) {
+      elsif( $membership->[0] eq "join" and !defined $membership->[1] ) {
+         # $member->displayname won't be set any more
          $self->push_displayevent( "matrix.leave" => {
-            name    => $member->displayname // "[$member_id]",
+            name    => $changes{displayname}[0] // $member->displayname,
             user_id => $member_id,
-         });
+         }, time => $tstamp );
          $self->bump_level( 1 );
-         return
       }
+      elsif( $membership->[1] eq "invite" ) {
+         $self->push_displayevent( "matrix.invite" => {
+            name    => $member->displayname,
+            user_id => $member_id,
+            invitee => $subject->displayname,
+            invitee_id => $subject->user->user_id,
+         }, time => $tstamp );
+         $self->bump_level( 1 );
+      }
+      else {
+         $self->push_displayevent( "matrix.member" => {
+            name    => $member->displayname,
+            user_id => $member_id,
+            subject => $subject->displayname // $subject->user->user_id,
+            state   => "membership",
+            oldval  => $membership->[0],
+            newval  => $membership->[1],
+         }, time => $tstamp );
+      }
+
+      return;
    }
 
    # As a text-only client we don't care about avatar_url
@@ -327,13 +365,14 @@ sub on_membership
       }, time => $tstamp );
       $self->bump_level( 1 );
    }
-   elsif( my $state = $changes{state} ) {
-      my $message = $changes{status_msg} && $changes{status_msg}[1];
-
-      $self->push_displayevent( "matrix.state" => {
-         state   => $state->[1],
-         message => ( defined $message ? "($message)" : "" ),
-         name    => $member->displayname // "[$member_id]",
+   elsif( my $level = $changes{level} ) {
+      $self->push_displayevent( "matrix.member" => {
+         name    => $member->displayname,
+         user_id => $member_id,
+         subject => $subject->displayname // $subject->user->user_id,
+         state   => "level",
+         oldval  => $level->[0],
+         newval  => $level->[1],
       }, time => $tstamp );
       $self->bump_level( 1 );
    }
@@ -347,25 +386,55 @@ sub on_membership
    }
 }
 
+my %key_for_event = (
+   "m.room.guest_access"       => "guest_access",
+   "m.room.history_visibility" => "history_visibility",
+   "m.room.join_rules"         => "join_rule",  # sic
+   "m.room.name"               => "name",
+   "m.room.topic"              => "topic",
+);
+
 sub on_state_changed
 {
    my $self = shift; shift;
    my ( $member, $event, %changes ) = @_;
    my $member_id = $member ? $member->user->user_id : undef;
 
+   my $type = $event->{type};
+
    $self->{latest_event_id} = $event->{event_id};
 
    my $watches = $self->{state_watches};
-   if( my $code = $watches->{ $event->{type} } ) {
+   if( my $code = $watches->{$type} ) {
       $code->( $self, $event );
    }
 
    my $tstamp = $event->{origin_server_ts} / 1000;
 
-   if( 1 ) {
+   # TODO: m.room.create event arrives before the m.room.member event for its
+   #   creator, so we don't yet have a way to display it
+   return if $type eq "m.room.create";
+
+   # As a text-only client we don't care about the room avatar
+   return if $type eq "m.room.avatar_url";
+
+   if( my $key = $key_for_event{$type} ) {
+      return unless my $values = $changes{$key};
+
+      $self->push_displayevent( "matrix.state" => {
+         name    => $member ? $member->displayname : undef,
+         user_id => $member_id,
+         state   => $key,
+         oldval  => $values->[0],
+         newval  => $values->[1],
+      }, time => $tstamp );
+
+      $self->bump_level( 1 );
+   }
+   else {
       # TODO for debugging
       $self->push_displayevent( text => {
-         text => "Member $member changed room state ${\pp \%changes}"
+         text => "Member $member changed room state $type to ${\pp \%changes}"
       }, time => $tstamp );
    }
 }
@@ -422,8 +491,10 @@ sub enter_text
    my $self = shift;
    my ( $text ) = @_;
 
+   my $content = build_formatted_message( parse_markdownlike( $text ) );
+
    my $room = $self->{room};
-   my $f = $room->send_message( $text );
+   my $f = $room->send_message( type => "m.text", %$content );
 
    $room->adopt_future( $f );
 }
@@ -448,6 +519,8 @@ sub command_say
    my $self = shift;
    my ( $text ) = @_;
 
+   # No markdown parsing
+
    $self->enter_text( $text );
 
    return;
@@ -460,8 +533,10 @@ sub command_me
    my $self = shift;
    my ( $text ) = @_;
 
+   my $content = build_formatted_message( parse_markdownlike( $text ) );
+
    my $room = $self->{room};
-   my $f = $room->send_message( type => "m.emote", body => $text );
+   my $f = $room->send_message( type => "m.emote", %$content );
 
    $room->adopt_future( $f );
 
